LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;

ENTITY ftdi_if IS
PORT (
	-- clock
	clock		: IN  std_logic;

	-- signal to tristate driver:
	tris_usb_out: OUT std_logic := '0';

	-- signal to FTDI-UART
	FTDI_nRD	: OUT std_logic := '1';
	FTDI_WR		: OUT std_logic := '0';

	-- signal from FTDI-UART
	FTDI_nRXF	: IN  std_logic;
	FTDI_nTXE	: IN  std_logic;

	-- control-signals
	load		: OUT std_logic := '0';
	state_0		: IN  std_logic
);
END ftdi_if;

-- For Timing info see:
-- FT245R Datasheet p.12-13

ARCHITECTURE a_ftdi_if OF ftdi_if IS


TYPE   STATE_TYPE IS	(S_IDLE, S_RD_START, S_RD_DELAY, S_RD_END, S_WR_START, S_WR_DELAY);

SIGNAL state    : STATE_TYPE;
SIGNAL delaycnt : integer range 0 to 15;


BEGIN

	PROCESS
	BEGIN
		WAIT UNTIL clock'EVENT AND clock='1';

		CASE state IS
			WHEN S_IDLE =>
				tris_usb_out<= '0';
				
				IF FTDI_nRXF = '0' THEN
					state       <= S_RD_START;
				END IF;

			WHEN S_RD_START =>
				FTDI_nRD    <= '0';
				delaycnt    <= 0;
				state       <= S_RD_DELAY;

			WHEN S_RD_DELAY =>
				IF delaycnt < 7 THEN -- generate a 80 ns delay..
					delaycnt    <= delaycnt + 1;
				ELSE
					load        <= '1'; -- fetch data now.
					delaycnt    <= 0;
					state       <= S_RD_END;
				END IF;

			WHEN S_RD_END =>
				load        <= '0';
				FTDI_nRD    <= '1';
				IF FTDI_nTXE = '0' THEN
					state       <= S_WR_START;
				END IF;

			WHEN S_WR_START =>
				FTDI_WR     <= state_0; -- only write data if state_0 is set.
				delaycnt    <= 0;
				state       <= S_WR_DELAY;

			WHEN S_WR_DELAY =>
				tris_usb_out<= state_0; -- only activate output if state_0 is set.
				IF delaycnt < 13 THEN -- also do RD_PRECHARGE... wait at least 130ns
					delaycnt    <= delaycnt + 1;
				ELSE
					FTDI_WR     <= '0';
					state       <= S_IDLE;
				END IF;


			WHEN OTHERS=>
				NULL;
		END CASE;
	END PROCESS;

END a_ftdi_if;
